/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
#ifndef WRTDEVICEAPIS_COMMONS_STRINGUTILS_H_
#define WRTDEVICEAPIS_COMMONS_STRINGUTILS_H_

#include <string>
#include <vector>

namespace WrtDeviceApis {
namespace Commons {
namespace String {

/**
 * Removes spaces from begining of the string.
 * @param str String to remove spaces from.
 * @return String with spaces removed.
 */
std::string lTrim(const std::string& str);

/**
 * Removes spaces from end of the string.
 * @param str String to remove spaces from.
 * @return String with spaces removed.
 */
std::string rTrim(const std::string& str);

/**
 * Removes spaces from begining and end of the string.
 * @param str String to remove spaces from.
 * @return String with spaces removed.
 */
std::string trim(const std::string& str);

/**
 * Checks if str starts with startStr
 * @param str String to be searched.
 * @param startStr String to be found at the beginning of str.
 * @return true when str starts with startStr, false otherwise.
 */
bool startsWith(const std::string& str,
        const std::string& startStr);

/**
 * Checks if str ends with startStr
 * @param str String to be searched.
 * @param endStr String to be found at the end of str.
 * @return true when str starts with startStr, false otherwise.
 */
bool endsWith(const std::string& str,
        const std::string& endStr);

/**
 * Merges consecutive groups of duplicate characters into one.
 * @param str String to remove characters from.
 * @param needle Character for which groups of duplicates to merge.
 * @return String with group of duplicate characters merged.
 * @code
 * ...
 * std::string str = "Aala maa kota";
 * std::string uniq = unique(str, 'a');
 * uniq == "Ala ma kota";
 * ...
 * @endcode
 */
std::string unique(const std::string& str,
        const char needle);

/**
 * Checks if supplied string exists in specified array of c-strings.
 * @param needle String to verify.
 * @param stack Array of c-strings which last element should be NULL.
 * @return True if string exists in array, false otherwise.
 * @remarks Last element of array should be NULL, otherwise there is no
 *          way to calculate its number of elements.
 */
bool inCStringArray(const std::string& needle,
        const char* stack[]);

/**
 * Duplicates a string.
 * @param str String to duplicate.
 * @return String copy.
 * @throw std::bad_alloc If allocation fails.
 * @remarks Ownership passed to the caller. String has to be deallocated using
 *          free() function.
 */
char* strdup(const char* str);
inline char* strdup(const std::string& str)
{
    return strdup(str.c_str());
}

std::string merge(const std::vector<std::string>& strs,
        std::string::value_type delim);

std::vector<std::string> split(const std::string& str,
        std::string::value_type delim);

int toInt(const std::string &text);

} // String
}
} // WrtDeviceApisCommon

#endif // WRTDEVICEAPIS_COMMONS_STRINGUTILS_H_
