/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
#ifndef WRTDEVICEAPIS_MMPLAYER_IMANAGER_H_
#define WRTDEVICEAPIS_MMPLAYER_IMANAGER_H_

#include <dpl/noncopyable.h>
#include <MMPlayer/EventOpen.h>
#include <MMPlayer/EventPlay.h>
#include <MMPlayer/EventPause.h>
#include <MMPlayer/EventResume.h>
#include <MMPlayer/EventStop.h>
#include <MMPlayer/EventIsPlaying.h>
#include <MMPlayer/EventSetWindow.h>
#include <MMPlayer/EventGetVolume.h>

namespace WrtDeviceApis {
namespace MMPlayer {
namespace Api {

class IManager :
    public Commons::EventRequestReceiver<EventOpen>,
    public Commons::EventRequestReceiver<EventPlay>,
    public Commons::EventRequestReceiver<EventPause>,
    public Commons::EventRequestReceiver<EventResume>,
    public Commons::EventRequestReceiver<EventStop>,
    public Commons::EventRequestReceiver<EventIsPlaying>,
    public Commons::EventRequestReceiver<EventSetWindow>,
    public Commons::EventRequestReceiver<EventGetVolume>
{
  public:
    static IManager &getInstance();
    virtual ~IManager();

    /**
     * Opens a file
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventOpen.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void openFile(const EventOpenPtr& event) = 0;
    /**
     * Plays current file. If file is already playing player starts from the beginning
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventPlay.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void play(const EventPlayPtr& event) = 0;
    /**
     * Pauses current playing file. If no file is currently playing noting happens
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventPause.
     * @exception Commons::PlatformException when platform error occurs
     * @exception Commons::InvalidArgumentException when fileName is empty
     */
    virtual void pause(const EventPausePtr& event) = 0;
    /**
     * Resumes after pause. If file is not paused nothing happens
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventResume.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void resume(const EventResumePtr& event) = 0;
    /**
     * Stops current playing file. If file is not playing nothing happens
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventStop.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void stop(const EventStopPtr& event) = 0;
    /**
     * Sets window for video files
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventSetWindow.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void setWindow(const EventSetWindowPtr& event) = 0;
    /**
     * Returns current playing state
     * @param event @see WrtDeviceApis::Api::MMPlayer::EventIsPlaying.
     * @return True if player is in playing state
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void isPlaying(const EventIsPlayingPtr& event) = 0;
    /**
     * Returns current playing state
     * @param [out] event @see WrtDeviceApis::Api::MMPlayer::EventIsPlaying.
     * @exception Commons::PlatformException when platform error occurs
     */
    virtual void getVolume(const EventGetVolumePtr& event) = 0;

  protected:
    IManager();

    virtual void OnRequestReceived(const EventOpenPtr& event) = 0;
    virtual void OnRequestReceived(const EventPlayPtr& event) = 0;
    virtual void OnRequestReceived(const EventPausePtr& event) = 0;
    virtual void OnRequestReceived(const EventResumePtr& event) = 0;
    virtual void OnRequestReceived(const EventStopPtr& event) = 0;
    virtual void OnRequestReceived(const EventIsPlayingPtr& event) = 0;
    virtual void OnRequestReceived(const EventSetWindowPtr& event) = 0;
    virtual void OnRequestReceived(const EventGetVolumePtr& event) = 0;
};
}
}
}

#endif // WRTDEVICEAPIS_MMPLAYER_IMANAGER_H_
